#include "Inspvaxa.h"
#include "functions.h"

INSStatus StringToINSStatus(const std::string &statusString)
{
    if (statusString == "INS_INACTIVE")
    {
        return INSStatus::INACTIVE;
    }
    else if (statusString == "INS_ALIGNING")
    {
        return INSStatus::ALIGNING;
    }
    else if (statusString == "INS_HIGH_VARIANCE")
    {
        return INSStatus::HIGH_VARIANCE;
    }
    else if (statusString == "INS_SOLUTION_GOOD")
    {
        return INSStatus::SOLUTION_GOOD;
    } // ... include other cases for different strings
    else if (statusString == "INS_SOLUTION_FREE")
    {
        return INSStatus::SOLUTION_FREE;
    }
    else
    {
        // Handle unknown status
        throw std::invalid_argument("Unknown INSStatus string");
    }
}

PositionType StringToPositionType(const std::string& typeString) {
    if (typeString == "NONE") {
        return PositionType::NONE;
    } else if (typeString == "FIXEDPOS") {
        return PositionType::FIXEDPOS;
    } else if (typeString == "FIXEDHEIGHT") {
        return PositionType::FIXEDHEIGHT;
    } // ... include other cases for different strings
    else if (typeString == "INS_RTKFLOAT") {
        return PositionType::INS_RTKFLOAT;
    } else if (typeString == "INS_RTKFIXED") {
        return PositionType::INS_RTKFIXED;
    } else {
        // Handle unknown type
        throw std::invalid_argument("Unknown PositionType string");
    }
}

// Convert INSStatus to PositionStatus
slam::PositionStatus ConvertINSStatusToPositionStatus(INSStatus status) {
    switch (status) {
        case INSStatus::SOLUTION_GOOD:
            return slam::PositionStatus::FIXED_3D; // Assuming SOLUTION_GOOD corresponds to 3D fixed position
        case INSStatus::ALIGNING:
        case INSStatus::HIGH_VARIANCE:
            return slam::PositionStatus::DR; // Assuming these correspond to Dead Reckoning
        // Add cases for other INSStatus values as needed
        default:
            return slam::PositionStatus::INVALID;
    }
}

// Convert PositionType to PositionQuality
slam::PositionQuality ConvertPositionTypeToPositionQuality(PositionType type) {
    switch (type) {
        case PositionType::INS_RTKFIXED:
            return slam::PositionQuality::RTK_FIXED;
        case PositionType::INS_RTKFLOAT:
            return slam::PositionQuality::RTK_FLOAT;
        case PositionType::SINGLE:
            return slam::PositionQuality::SINGLE;
        // Add cases for other PositionType values as needed
        default:
            return slam::PositionQuality::INVALID;
    }
}

// Function to map INSPVAXA to InsData
slam::InsData InspvaxaData::MapINSPVAXAToInsData(const INSPVAXA& inp)
{
    slam::InsData out;

    // Assuming that the utc_timestamp can be derived from the header's week and seconds
    out.utc_timestamp = static_cast<uint64_t>(inp.header.week) * 604800*1000 + static_cast<uint64_t>(inp.header.seconds*1000);

    out.latitude = inp.latitude;
    out.longitude = inp.longitude;
    out.ellipsoid = inp.undulation; // If ellipsoid is not directly present, it may need to be calculated or set to a default value.

    out.position_status = ConvertINSStatusToPositionStatus(inp.ins_status);
    out.position_quality = ConvertPositionTypeToPositionQuality(inp.pos_type);

    out.altitude = static_cast<float>(inp.height);

    // Velocity mapping
    out.velocity = {static_cast<float>(inp.north_velocity), static_cast<float>(inp.east_velocity), static_cast<float>(inp.up_velocity)};

    // Ground velocity might need to be computed if it's not directly available
    // out.ground_vel_2d = ComputeGroundVelocity2D(inp);

    // Map attitude (roll, pitch, azimuth) to INS_Attitude
    out.attitude = {static_cast<float>(inp.azimuth), static_cast<float>(inp.roll), static_cast<float>(inp.pitch)};

    out.heading = static_cast<float>(inp.azimuth);

    // Errors and other parameters would be mapped similarly, e.g.:
    out.vertical_velocity_error = inp.roll_std_dev;
    // ... and so on for other fields

    return out;
}

INSPVAXA InspvaxaData::parseINSPVAXA(std::string data)
{

    INSPVAXA insData;

    auto fields = Functions::splitWithStl(data, ","); // Split the data string by comma

    if(fields.size() < 32)return insData;

    insData.header.week = convertTo<uint32_t>(fields[5]);
    insData.header.seconds = convertTo<double>(fields[6]);

    auto field9 = Functions::splitWithStl(fields[9], ";"); // Split the data string by comma

    insData.ins_status = StringToINSStatus(field9[1]);
    insData.pos_type = StringToPositionType(fields[10]);

    insData.latitude = convertTo<double>(fields[11]);
    insData.longitude = convertTo<double>(fields[12]);
    insData.height = convertTo<double>(fields[13]);
    insData.undulation = convertTo<float>(fields[14]);
    insData.north_velocity = convertTo<double>(fields[15]);
    insData.east_velocity = convertTo<double>(fields[16]);
    insData.up_velocity = convertTo<double>(fields[17]);
    insData.roll = convertTo<double>(fields[18]);
    insData.pitch = convertTo<double>(fields[19]);
    insData.azimuth = convertTo<double>(fields[20]);
    insData.latitude_std_dev = convertTo<float>(fields[21]);
    insData.longitude_std_dev = convertTo<float>(fields[22]);
    insData.height_std_dev = convertTo<float>(fields[23]);


    return insData;

    insData.north_velocity_std_dev = convertTo<float>(fields[24]);
    insData.east_velocity_std_dev = convertTo<float>(fields[25]);
    insData.up_velocity_std_dev = convertTo<float>(fields[26]);
    insData.roll_std_dev = convertTo<float>(fields[27]);
    insData.pitch_std_dev = convertTo<float>(fields[28]);
    insData.azimuth_std_dev = convertTo<float>(fields[29]);
    insData.extended_solution_status = convertTo<uint32_t>(fields[30]);
    insData.time_since_update = convertTo<uint32_t>(fields[31]);

    return insData;
}
